import { Component, Directive, ElementRef, Input, NgModule, Renderer2, forwardRef } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule, NG_VALIDATORS, NG_VALUE_ACCESSOR } from '@angular/forms';

class DateUtils {
    /**
     * @param {?} myDate
     * @param {?} myTimezone
     * @return {?}
     */
    static adjustTimezone(myDate, myTimezone) {
        const /** @type {?} */ offset = isNaN(myTimezone) ? new Date().getTimezoneOffset() : parseFloat(myTimezone) * 60;
        return new Date(myDate.getTime() + offset * 60 * 1000);
    }
    /**
     * @param {?} num
     * @return {?}
     */
    static parseIntStrict(num) {
        return (num !== null && num !== '' && !isNaN(parseInt(num, 10))) ? parseInt(num, 10) : null;
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    static getMaxDate(month, year) {
        let /** @type {?} */ res = 31;
        if (month != null) {
            if (month === 4 || month === 6 || month === 9 || month === 11) {
                res = 30;
            }
            if (year !== null && month === 2) {
                res = ((year % 4 === 0) && (year % 100 !== 0)) || (year % 400 === 0) ? 29 : 28;
            }
        }
        return res;
    }
    /**
     * @param {?} myDate
     * @param {?} myTimezone
     * @return {?}
     */
    static parseDate(myDate, myTimezone) {
        let /** @type {?} */ res = null;
        if (myDate !== undefined && myDate !== null) {
            if (myDate instanceof Date) {
                res = myDate;
            }
            else {
                if (typeof myDate === 'number' || typeof myDate === 'string') {
                    // Parse date.
                    if (typeof myDate === 'number') {
                        res = new Date(parseInt(myDate.toString(), 10));
                    }
                    else {
                        res = new Date(myDate);
                    }
                    // Adjust timezone.
                    res = DateUtils.adjustTimezone(res, myTimezone);
                }
            }
        }
        return res;
    }
}

const DEFAULT_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(() => ComboDatepickerComponent),
    multi: true
};
const DEFAULT_VALIDATOR = {
    provide: NG_VALIDATORS,
    useExisting: forwardRef(() => ComboDatepickerComponent),
    multi: true,
};
class ComboDatepickerComponent {
    constructor() {
        this.attrsDate = {};
        this.attrsMonth = {};
        this.attrsYear = {};
        this.visible = [true, true, true];
        this.selects = {
            d: { visible: true },
            m: { visible: true },
            y: { visible: true }
        };
        this.monthNames = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
        this.propagateChange = () => { };
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        // Initialize model.
        this.ngModel = DateUtils.parseDate(this.ngModel, this.timezone);
        // Initialize attributes variables.
        this.selects.d.attrs = this.attrsDate;
        this.selects.m.attrs = this.attrsMonth;
        this.selects.y.attrs = this.attrsYear;
        if (typeof this.disabled === 'boolean') {
            this.selects.d.disabled = this.disabled;
            this.selects.m.disabled = this.disabled;
            this.selects.y.disabled = this.disabled;
        }
        else if (this.disabled instanceof Array && this.disabled.length === 3) {
            this.selects.d.disabled = this.disabled[0];
            this.selects.m.disabled = this.disabled[1];
            this.selects.y.disabled = this.disabled[2];
        }
        if (typeof this.visible === 'boolean') {
            this.selects.d.visible = this.visible;
            this.selects.m.visible = this.visible;
            this.selects.y.visible = this.visible;
        }
        else if (this.visible instanceof Array && this.visible.length === 3) {
            this.selects.d.visible = this.visible[0];
            this.selects.m.visible = this.visible[1];
            this.selects.y.visible = this.visible[2];
        }
        // Verify if initial date was defined.
        const /** @type {?} */ initDate = DateUtils.parseDate(this.date, this.timezone);
        if (initDate != null) {
            this.ngModel = initDate;
        }
        // Initialize order.
        if (typeof this.order !== 'string') {
            this.order = 'dmy'.split('');
        }
        else {
            this.order = this.order.toLowerCase().split('');
        }
        // Initialize minimal and maximum values.
        this._minDate = DateUtils.parseDate(this.minDate, this.timezone);
        if (this.minDate == null) {
            const /** @type {?} */ now = new Date();
            this._minDate = new Date(now.getFullYear() - 100, now.getMonth(), now.getDate(), now.getHours(), now.getMinutes(), now.getSeconds(), now.getMilliseconds());
        }
        this._maxDate = DateUtils.parseDate(this.maxDate, this.timezone);
        if (this._maxDate == null) {
            this._maxDate = new Date();
        }
        // Verify if selected date is in the valid range.
        if (this.ngModel && this.ngModel < this._minDate) {
            this.ngModel = this._minDate;
        }
        if (this.ngModel && this.ngModel > this._maxDate) {
            this.ngModel = this._maxDate;
        }
        if (this.months !== undefined && this.months !== null) {
            if (typeof this.months === 'string') {
                const /** @type {?} */ months = this.months.split(',');
                if (months.length === 12) {
                    this.monthNames = months;
                }
            }
            if (Array.isArray(this.months) && this.months.length === 12) {
                this.monthNames = this.months;
            }
        }
        this.updatePlaceholders();
        this.updateMonthList('');
        this.updateYearList('');
        this.updateDateList('', '');
        this.writeValue(this.ngModel);
    }
    /**
     * @return {?}
     */
    onChange() {
        let /** @type {?} */ res = null;
        // Check that the three combo boxes have values.
        if (this.selects.d.value && !isNaN(this.selects.m.value) &&
            this.selects.y.value != null && this.selects.y.value) {
            const /** @type {?} */ maxDay = DateUtils.getMaxDate(this.selects.m.value + 1, this.selects.m.value);
            let /** @type {?} */ hours = 0;
            let /** @type {?} */ minutes = 0;
            let /** @type {?} */ seconds = 0;
            let /** @type {?} */ milliseconds = 0;
            if (this.ngModel != null) {
                hours = this.ngModel.getHours();
                minutes = this.ngModel.getMinutes();
                seconds = this.ngModel.getSeconds();
                milliseconds = this.ngModel.getMilliseconds();
            }
            // Remove already selected day & date if day is  greater than maxday then form invalid value also resets
            if (this.selects.d.value > maxDay) {
                this.selects.d.value = null;
            }
            else {
                res = new Date(this.selects.y.value, this.selects.m.value, this.selects.d.value > maxDay ? maxDay : this.selects.d.value, hours, minutes, seconds, milliseconds);
            }
        }
        // Disable placeholders after selecting a value.
        if (this.placeHolders) {
            if (this.selects.y.value) {
                this.placeHolders[0].disabled = true;
            }
            if (this.selects.m.value) {
                this.placeHolders[1].disabled = true;
            }
            if (this.selects.d.value) {
                this.placeHolders[2].disabled = true;
            }
        }
        // Hide or show days and months according to the min and max dates.
        this.updateMonthList(this.selects.y.value);
        this.updateYearList(this.selects.m.value);
        this.updateDateList(this.selects.m.value, this.selects.y.value);
        this.propagateChange(res);
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value) {
            this.selects.d.value = value.getDate();
            this.selects.m.value = value.getMonth();
            this.selects.y.value = value.getFullYear();
        }
        else {
            if (typeof this.visible === 'boolean') {
                this.selects.d.value = this.visible ? null : 1;
                this.selects.m.value = this.visible ? null : 0;
                this.selects.y.value = this.visible ? null : new Date().getFullYear();
            }
            else if (this.visible instanceof Array && this.visible.length === 3) {
                this.selects.d.value = this.visible[0] ? null : 1;
                this.selects.m.value = this.visible[1] ? null : 0;
                this.selects.y.value = this.visible[2] ? null : new Date().getFullYear();
            }
            if (this.placeHolders) {
                this.placeHolders[0].disabled = false;
                this.placeHolders[1].disabled = false;
                this.placeHolders[2].disabled = false;
            }
        }
        this.updateMonthList(this.selects.y.value);
        this.updateYearList(this.selects.m.value);
        this.updateDateList(this.selects.m.value, this.selects.y.value);
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this.propagateChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
    }
    /**
     * @param {?} c
     * @return {?}
     */
    validate(c) {
        return this.selects.d.value && this.selects.d.value && this.selects.y.value ?
            null : { required: true };
    }
    /**
     * @param {?} changes
     * @return {?}
     */
    ngOnChanges(changes) {
        if (changes.placeholder && !changes.placeholder.isFirstChange()) {
            this.updatePlaceholders();
            this.updateMonthList(this.selects.y.value);
            this.updateYearList(this.selects.m.value);
            this.updateDateList(this.selects.m.value, this.selects.y.value);
        }
    }
    /**
     * @param {?} year
     * @return {?}
     */
    updateMonthList(year) {
        // Parse parameter.
        year = DateUtils.parseIntStrict(year);
        // Some months can not be chosen if the year matches with the year of the minimum or maximum dates.
        const /** @type {?} */ start = year !== null && year === this._minDate.getFullYear() ? this._minDate.getMonth() : 0;
        const /** @type {?} */ end = year !== null && year === this._maxDate.getFullYear() ? this._maxDate.getMonth() : 11;
        // Generate list.
        this.selects.m.options = [];
        if (this.placeHolders) {
            this.selects.m.options.push(this.placeHolders[1]);
        }
        for (let /** @type {?} */ i = start; i <= end; i++) {
            this.selects.m.options.push({ value: i, name: this.monthNames[i] });
        }
    }
    /**
     * @param {?=} month
     * @return {?}
     */
    updateYearList(month) {
        // Parse parameter.
        month = DateUtils.parseIntStrict(month);
        this.selects.y.options = [];
        const /** @type {?} */ isReverse = typeof this.yearOrder === 'string' && this.yearOrder.indexOf('des') === 0;
        for (let /** @type {?} */ i = this._minDate.getFullYear(); i <= this._maxDate.getFullYear(); i++) {
            const /** @type {?} */ now = new Date();
            if (month + 1 < this._minDate.getMonth() && now.getFullYear() === i && month !== null &&
                (isReverse && month > now.getMonth() || !isReverse && month < now.getMonth())) {
                continue;
            }
            this.selects.y.options.push({ value: i, name: i });
        }
        // Verify if the order of the years must be reversed.
        if (isReverse) {
            this.selects.y.options.reverse();
        }
        // Prepend the years placeholder
        if (this.placeHolders) {
            this.selects.y.options.unshift(this.placeHolders[0]);
        }
    }
    /**
     * @param {?} month
     * @param {?} year
     * @return {?}
     */
    updateDateList(month, year) {
        // Parse parameters.
        month = DateUtils.parseIntStrict(month);
        year = DateUtils.parseIntStrict(year);
        // Start date is 1, unless the selected month and year matches the minimum date.
        let /** @type {?} */ start = 1;
        if (month !== null && month === this._minDate.getMonth() &&
            year !== null && year === this._minDate.getFullYear()) {
            start = this._minDate.getDate();
        }
        // End date is 30 or 31 (28 or 29 in February), unless the selected month and year matches the maximum date.
        let /** @type {?} */ end = DateUtils.getMaxDate(month !== null ? (month + 1) : null, year);
        if (month !== null && month === this._maxDate.getMonth() &&
            year !== null && year === this._maxDate.getFullYear()) {
            end = this._maxDate.getDate();
        }
        // Generate list.
        this.selects.d.options = [];
        if (this.placeHolders) {
            this.selects.d.options.push(this.placeHolders[2]);
        }
        for (let /** @type {?} */ i = start; i <= end; i++) {
            this.selects.d.options.push({
                value: i,
                name: i
            });
        }
    }
    /**
     * @return {?}
     */
    updatePlaceholders() {
        this.placeHolders = null;
        if (this.placeholder !== undefined && this.placeholder !== null &&
            (typeof this.placeholder === 'string' || Array.isArray(this.placeholder))) {
            const /** @type {?} */ holders = typeof this.placeholder === 'string' ?
                this.placeholder.split(',') : this.placeholder;
            if (holders.length === 3) {
                this.placeHolders = [];
                for (const /** @type {?} */ h of holders) {
                    this.placeHolders.push({ value: null, name: h, disabled: false });
                }
            }
        }
    }
}
ComboDatepickerComponent.decorators = [
    { type: Component, args: [{
                selector: 'ngx-combo-datepicker',
                template: `
    <ng-container *ngFor="let type of order">
      <select *ngIf="selects[type].visible"
              [(ngModel)]="selects[type].value"
              (change)="onChange($event)"
              [disabled]="selects[type].disabled"
              customAttrs [attrs]="selects[type].attrs">
        <option *ngFor="let option of selects[type].options"
                [ngValue]="option.value"
                [disabled]="option.disabled">
          {{ option.name }}
        </option>
      </select>
    </ng-container>
  `,
                providers: [
                    DEFAULT_VALUE_ACCESSOR,
                    DEFAULT_VALIDATOR
                ]
            },] },
];
/**
 * @nocollapse
 */
ComboDatepickerComponent.ctorParameters = () => [];
ComboDatepickerComponent.propDecorators = {
    'ngModel': [{ type: Input },],
    'date': [{ type: Input },],
    'minDate': [{ type: Input },],
    'maxDate': [{ type: Input },],
    'months': [{ type: Input },],
    'order': [{ type: Input },],
    'attrsDate': [{ type: Input },],
    'attrsMonth': [{ type: Input },],
    'attrsYear': [{ type: Input },],
    'yearOrder': [{ type: Input },],
    'timezone': [{ type: Input },],
    'placeholder': [{ type: Input },],
    'ngRequired': [{ type: Input },],
    'disabled': [{ type: Input },],
    'visible': [{ type: Input },],
};

class AttributesDirective {
    /**
     * @param {?} element
     * @param {?} renderer
     */
    constructor(element, renderer) {
        this.element = element;
        this.renderer = renderer;
    }
    /**
     * @return {?}
     */
    ngOnInit() {
        Object.keys(this.attrs).forEach((key) => this.renderer.setAttribute(this.element.nativeElement, key, this.attrs[key]));
    }
}
AttributesDirective.decorators = [
    { type: Directive, args: [{
                selector: '[customAttrs]'
            },] },
];
/**
 * @nocollapse
 */
AttributesDirective.ctorParameters = () => [
    { type: ElementRef, },
    { type: Renderer2, },
];
AttributesDirective.propDecorators = {
    'attrs': [{ type: Input },],
};

class ComboDatepickerModule {
}
ComboDatepickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [
                    ComboDatepickerComponent,
                    AttributesDirective,
                ],
                exports: [
                    ComboDatepickerComponent
                ]
            },] },
];
/**
 * @nocollapse
 */
ComboDatepickerModule.ctorParameters = () => [];

/**
 * Generated bundle index. Do not edit.
 */

export { ComboDatepickerModule, ComboDatepickerComponent as ɵc, DEFAULT_VALIDATOR as ɵb, DEFAULT_VALUE_ACCESSOR as ɵa, AttributesDirective as ɵd };
//# sourceMappingURL=ngx-combo-datepicker.js.map
